﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Assets;
using UnityEngine;
using UnityStandardAssets.Characters.ThirdPerson;

public class AnimatedModelToRagdoll : MonoBehaviour
{
    void OnCollisionEnter(Collision collision)
    {
        // check this isnt the ground
        var thisRoot = GetComponentInParent<ModelPhysicsSetup>();
        if (thisRoot != null && thisRoot.gameObject.tag == "Ground")
            return;

        // check other model has a phyiscs handler on it's root
        var other = collision.gameObject;
        var otherRoot = other.GetComponentInParent<ModelPhysicsSetup>();
        if (otherRoot == null || !otherRoot.CanRagdoll)
            return;

        // dont want to affect the players physics
        var player = otherRoot.GetComponentInParent<ThirdPersonCharacter>();
        if (player != null)
            return;

        // if other model is animating, lets turn it into a ragdoll
        if (otherRoot.IsKinematic)
        {
            // disable waypoint nav
            var waypointNav = otherRoot.GetComponent<MoveAlongWaypointSystem>();
            if (waypointNav != null)
                waypointNav.enabled = false;

            // turn on physics
            Utils.ToggleIsKinematic(otherRoot.gameObject);
            Utils.SetRenderersToUpdateWhenOffScreen(otherRoot.gameObject);

            // disable animation
            var animator = otherRoot.GetComponent<Animator>();
            if (animator != null)
                animator.enabled = false;

            // repulse
            foreach (ContactPoint contact in collision.contacts)
            {
                Debug.DrawRay(contact.point, contact.normal, Color.white);

                var impulseToApply =
                    contact.normal *
                    Mathf.Abs(contact.separation) *
                    (contact.thisCollider.attachedRigidbody.velocity.magnitude / 2) *
                    -1;

                contact.otherCollider.attachedRigidbody.AddForce(impulseToApply, ForceMode.Impulse);

                // also apply impulse to source model, if it is not animated
                if (thisRoot != null && !thisRoot.IsKinematic)
                {
                    var oppImpulseToApply =
                        contact.normal *
                        Mathf.Abs(contact.separation) *
                        (contact.otherCollider.attachedRigidbody.velocity.magnitude / 2);

                    contact.thisCollider.attachedRigidbody.AddForce(oppImpulseToApply, ForceMode.Impulse);
                }
            }

            otherRoot.IsKinematic = false;
        }
    }
}
