﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Assets;
using UnityEngine;
using UnityStandardAssets.Characters.ThirdPerson;

public class ModelPhysicsSetup : MonoBehaviour {

    private const float MAX_DEPENETRATION_VELOCITY = 1f;

    public string RigidBodyIgnoreGroupsJSON;

    internal bool IsKinematic;
    internal bool CanRagdoll;

	// Use this for initialization
	void Start  () {
     
        var ignoreGroups = JsonUtility.FromJson<RagdollBoneIgnoreGroups>(RigidBodyIgnoreGroupsJSON);
        
        var boneLookup = Utils.GetColliders(gameObject)
            .ToDictionary(c => c.transform.parent.name, c => c);

        for (int i = 0; i < ignoreGroups.Groups.Length; i++)
        {
            var group = ignoreGroups.Groups[i];

            Collider boneCollider;
            if (!boneLookup.TryGetValue(group.BoneName, out boneCollider))
                continue;
        
            for (int j = 0; j < group.IgnoreBoneNames.Length; j++)
            {
                Collider boneCollider2;
                if (!boneLookup.TryGetValue(group.IgnoreBoneNames[j], out boneCollider2))
                    continue;
                
                Physics.IgnoreCollision(boneCollider, boneCollider2, true);
            }

            // set maxDepenetrationVelocity
            var rigidBody = boneCollider.transform.parent.GetComponent<Rigidbody>();
            if (rigidBody != null)
            {
                rigidBody.maxDepenetrationVelocity = MAX_DEPENETRATION_VELOCITY;
                rigidBody.maxAngularVelocity = MAX_DEPENETRATION_VELOCITY;
            }
        }

        // set maxDepenetrationVelocity on controller
        var player = GetComponentInParent<ThirdPersonCharacter>();
        if (player != null)
        {
            var rigidBody = player.GetComponent<Rigidbody>();
            rigidBody.maxDepenetrationVelocity = MAX_DEPENETRATION_VELOCITY;
            rigidBody.maxAngularVelocity = MAX_DEPENETRATION_VELOCITY;
        }

        // get a bool if model is animated or not
        var animator = GetComponent<Animator>();
        IsKinematic = (animator != null && animator.runtimeAnimatorController != null);

        CanRagdoll = (GetComponentInChildren<AnimatedModelToRagdoll>() != null);
    }
    
    [Serializable]
    public class RagdollBoneIgnoreGroups
    {
        public RagdollBoneIgnoreGroup[] Groups;
    }

    [Serializable]
    public class RagdollBoneIgnoreGroup
    {
        public string BoneName;
        public string[] IgnoreBoneNames;
    }
}
