﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityStandardAssets.Utility;

public class MoveAlongWaypointSystem : MonoBehaviour
{
    [SerializeField]
    public WaypointProgressTracker waypointSystem;

    [SerializeField]
    public float speed = 1f;

    [SerializeField]
    public float rotationOffset = 0;

    private Quaternion rotationOffsetQuat;
    
    void Start()
    {
        rotationOffsetQuat = Quaternion.AngleAxis(rotationOffset, Vector3.up);
    }
    
    void Update()
    {
        var direction = GetDirection();

        if (direction != Vector3.zero)
        {
            transform.position += (direction * speed);

            var rotationSpeed = speed * 2;

            transform.rotation = Quaternion.Slerp(
                transform.rotation,
                Quaternion.LookRotation(direction) * rotationOffsetQuat,
                Time.deltaTime * rotationSpeed);
        };
    }

    public Vector3 GetDirection()
    {
        var direction = waypointSystem.target.position - transform.position;
        direction.y = 0;
        direction.Normalize();
        return direction;
    }
}
